/*
Marching To The Sirens Of War
Lee Zakian
2/28/05

========================================================
INSTRUMENTS	SIGNAL FLOW	
	
			iout			Mixer
1	Gliss 1		1			100	Channel 1/Out: L-R
2	Gliss 2		2			100	Channel 2/Out: L-R
3	Gliss 3		3			100	Channel 3/Out: L-R
			
	2 Part Canon		EQ	Reverb	Mixer	PANNED OUTPUT
4	Flute 1		4	92	97-L	100	Channel 4/Out: L-R
5	Flute 2		5	93	97-R	100	Channel 5/Out: L-R
	
6	Bass		6	94	98	100	Channel 6/Out: L-R
========================================================
*/

	sr = 44100
	kr = 4410
	ksmps = 10
	nchnls = 2
	zakinit	 	20, 20

	instr 1
;==========
;|||||||||| 	Gliss 1
;==========
;Pitch/Amplitude
kglis	expseg	220, p3/5, 440, p3/5, 880, p3/5, 110, p3/5, 440, p3/5, 220
kamp  	linseg	100, p3/5, 200, p3/5, 500, p3/5, 200, p3/5, 100, p3/5, 50
k1	linen	kamp, p10, p3, p11	;attack release envelope
k2	expon	p8, p3, p9		;harmonic envelope

;Reverb
irvbtime	=	p4
iwet		=	p5
idry		=	1-p5
ihfroll		=	p6	;low pass filter: high frequency rolloff
ihfdiff		=	p7	;high frequency diffusion
iout		=	1	;Mixer channel, with Gliss 2

a1	oscili	k1, (k2+1)*kglis, 1	;table 1
arevL	nreverb	(a1*1.5)*iwet, p4, p7
aout	tone	arevL, p6

;Pan
kpan	linseg	1, p3/5, .7, p3/5, .3, p3/5, 0, p3/5, .3, p3/5, .9
kpanL	=	kpan
kpanR	=	1-kpan
	zawm		aout*(kpanL+kpanR), iout
	endin

	instr 2
;==========
;|||||||||| 	Gliss 2
;==========
;Pitch/Amplitude
kglis2	expseg	110, p3/9, 440, p3/9, 220, p3/9, 880, p3/9, 220, p3/9, 440, p3/9, 110, p3/9, 220, p3/9, 440, p3/9, 55
kamp  	linseg	100, p3/5, 200, p3/5, 500, p3/5, 200, p3/5, 100, p3/5, 50
k1	linen	kamp, p10, p3, p11	;attack release envelope
k2	expon	p8, p3, p9		;harmonic envelope

;Reverb
irvbtime	=	p4
iwet		=	p5
idry		=	1-p5
ihfroll		=	p6	;low pass filter: high frequency rolloff
ihfdiff		=	p7	;high frequency diffusion
iout		=	2	;Mixer channel, with Gliss 1

a2	oscili	k1, (k2+1)*kglis2, 2	;table 2
arevR	nreverb	(a2*1.5)*iwet, p4, p7
aout	tone	arevR, p6

;Pan
kpan	linseg	0, p3/5, .2, p3/5, .6, p3/5, 1, p3/5, .5, p3/5, .2
kpanL	=	kpan
kpanR	=	1-kpan
	zawm		aout*(kpanL+kpanR), iout
	endin

	instr 3
;==========
;|||||||||| 	Gliss 3
;==========
;Pitch/Amplitude
kglis3	linseg	50, p3/4, 200, p3/4, 100, p3/4, 300, p3/4, 50
kglis4	linseg	60, p3/6, 300, p3/6, 500, p3/6, 700, p3/6, 300, p3/6, 100, p3/6, 60
kamp  	linseg	500, p3/3, 200, p3/3, 500, p3/3, 100
k1	linen	kamp, p10, p3, p11	;attack release envelope
k2	expon	p8, p3, p9		;harmonic envelope

;Reverb
irvbtime	=	p4
iwet		=	p5
idry		=	1-p5
ihfroll		=	p6	;low pass filter: high frequency rolloff
ihfdiff		=	p7	;high frequency diffusion
iout		=	3	;to Mixer channel

a1	oscili	k1, (k2+1)*kglis3, 1, 3	;table 3
a2	oscili	k1, (k2+1)*kglis4, 1, 4	;table 4
arev1	nreverb	(a1*1.5)*iwet, p4, p7
arev2	nreverb	(a2*1.5)*iwet, p4, p7
aout1	tone	arev1, p6
aout2	tone	arev2, p6
	zawm	aout1+aout2, iout
	endin

	instr 4
;==========
;|||||||||| 	Flute 1		Range of Flute		B3-D7	(7.11-11.02)
;========== 	Range of Piece		D4-C6	(8.02-10.00)
idur    	=       	abs(p3)	;absolute duration value
ipch1   =       	cpspch(p6)
ipch2   =       	cpspch(p5) 
kpch    	=       	ipch2
iport   	=       	(p6=0 ? 0.003 : 0.5)
	    	;portamento:glissando from P6 pitch, minimal pitch ramp when P6=0
iamp    	=       	p4	;tied note starts at amp ramp
;=====================
;VARIABLE TABLES
;=====================
iatkfn	=	26	;attack	
idecfn	=	28	;decay
icutfn	=	31	;high partials trim
ibrthfn	=	29	;breath
ivdfn	=	30	;vibrato depth
;=====================
;ATTACK
;=====================   
ioctat	=	octpch(p5)
iminat	=	octpch(10.00)
imaxat	=	octpch(8.02)
irngat	=	iminat-imaxat
indxat	=	(ioctat-iminat)/irngat
iatt	tablei	indxat, iatkfn, 1
iatt	=	(p3<0.4 ? (p3*0.1) : iatt)
;=====================
;DECAY
;=====================
ioctdc	=	octpch(p5)
imndc	=	octpch(10.00)
imxdc	=	octpch(8.02)
irngdc	=	imndc-imxdc
indxd	=	idur
indxdc	=	indxd+(ioctdc-imndc)/irngdc
idec	tablei	indxdc, idecfn, 1
idec	=	(p3<0.4 ? (p3*0.1) : idec)
;=====================
;PEAK/SWELL
;=====================
iout	=	4	;to EQ Instr 92
irise   	=       	idur*p8  ;0.1 = decresendo, 0.9 = crescendo, 0.5 = peak in middle
i1      	=       	-1         	;tied note phase
i2      	=       	-1          ;vibrato phase
kdclk	linseg  	0, .002, 1, p3-.004, 1, .002, 0	;declick envelope
;=====================
;AMPLITUDE RAMP 
;=====================
amp     		linseg  	iamp, iatt, iamp, idur-0.7, iamp, idec, iamp
;=====================
;PITCH
;=====================
pitch:
kpramp	linseg  	ipch1, iport, ipch2, idur-iport, ipch2
kpch  	=       	kpramp
;=====================
;EXPRESSION 
;rise/fall, vibrato
;=====================
expr:
irise	=	(p8 >0 ? irise : iatt)
ifall	=	idur-irise
p7	=	((p7+p4) >0 ? p7 : -p4)
aslur   	linseg  	0, irise, p7, ifall, 0	;make vibrato
;=====================
;VIBRATO DEPTH
;=====================
ioctvd	=	octpch(p5)
iminvd	=	octpch(8.02)
imaxvd	=	octpch(10.00)
irngvd	=	iminvd-imaxvd
indxvd	=	(ioctvd-iminvd)/irngvd
ivibd	tablei	indxvd, ivdfn, 1
avib	oscili	ivibd*2.5, p9, 25, 0.25	;vibrato depth, speed, table, phase
avib	=	avib+0.5
;=====================
;BREATH
;=====================
ioctbr	=	octpch(p5)
imnbr	=	octpch(8.02)
imxbr	=	octpch(10.00)
irngbr	=	imnbr-imxbr
indxbr	=	(ioctbr-imnbr)/irngbr
ibrth	tablei		indxbr, ibrthfn, 1
kenv1	linseg		0, .06, .5, .2, .9, p3-.16, .5, .02, 0
kenv2	linseg		0, .001, .1, p3-.002, .1, .001, 0
aflow	pinkish	kenv1*(p7*.02)
asum	=	(ibrth+aflow+kenv1)*avib
aslur   	=       	(aslur*avib)+asum
;=====================
;HIGH PARTIAL TRIM
;===================== 
ioct	=	octpch(p5)
imin	=	octpch(8.02)
imax	=	octpch(10.00)
irange	=	imin-imax
index	=	(ioct-imin)/irange
icut	tablei	index, icutfn, 1
;=====================
;PLAY 
;output to EQ instr 92
;=====================
play:					  ;waveform tables
aflu1	oscil3  		(amp+aslur), kpch, 5, i1		
aflu2	oscil3  		(amp+aslur), kpch, 6, i1
aflu3	oscil3  		(amp+aslur), kpch, 7, i1
aflu4	oscil3  		(amp+aslur), kpch, 8, i1
aflu5	oscil3  		(amp+aslur), kpch, 9, i1
aflu6	oscil3  		(amp+aslur), kpch, 10, i1
asig   	butterlp 	(aflu1+aflu2+aflu3+aflu4+aflu5+aflu6), icut, 1
	zawm		(asig*2.5)+asum*kenv2*kdclk, iout

	endin

	instr 5
;==========
;|||||||||| 	Flute 2		Range of Flute		B3-D7	(7.11-11.02)
;========== 	Range of Piece		????
idur    	=       	abs(p3)	;absolute duration value
ipch1   =       	cpspch(p6)
ipch2   =       	cpspch(p5)
kpch    	=       	ipch2
iport   	=       	(p6=0 ? 0.003 : 0.5)
	    	;portamento:glissando from P6 pitch, minimal pitch ramp when P6=0
iamp    	=       	p4	;tied note starts at amp ramp
;=====================
;VARIABLE TABLES
;=====================
iatkfn	=	26	;attack	
idecfn	=	28	;decay
icutfn	=	31	;high partials trim
ibrthfn	=	29	;breath
ivdfn	=	30	;vibrato depth
;=====================
;ATTACK
;=====================   
ioctat	=	octpch(p5)
iminat	=	octpch(10.00)
imaxat	=	octpch(8.02)
irngat	=	iminat-imaxat
indxat	=	(ioctat-iminat)/irngat
iatt	tablei	indxat, iatkfn, 1
iatt	=	(p3<0.4 ? (p3*0.1) : iatt)
;=====================
;DECAY
;=====================
ioctdc	=	octpch(p5)
imndc	=	octpch(10.00)
imxdc	=	octpch(8.02)
irngdc	=	imndc-imxdc
indxd	=	idur
indxdc	=	indxd+(ioctdc-imndc)/irngdc
idec	tablei	indxdc, idecfn, 1
idec	=	(p3<0.4 ? (p3*0.1) : idec)
;=====================
;PEAK/SWELL
;=====================
iout	=	5	;to EQ Instr 92
irise   	=       	idur*p8  ;0.1 = decresendo, 0.9 = crescendo, 0.5 = peak in middle
i1      	=       	-1         	;tied note phase
i2      	=       	-1          ;vibrato phase
kdclk	linseg  	0, .002, 1, p3-.004, 1, .002, 0	;declick envelope
;=====================
;AMPLITUDE RAMP 
;=====================
amp     		linseg  	iamp, iatt, iamp, idur-0.7, iamp, idec, iamp
;=====================
;PITCH
;=====================
pitch:
kpramp	linseg  	ipch1, iport, ipch2, idur-iport, ipch2
kpch  	=       	kpramp
;=====================
;EXPRESSION 
;rise/fall, vibrato
;=====================
expr:
irise	=	(p8 >0 ? irise : iatt)
ifall	=	idur-irise
p7	=	((p7+p4) >0 ? p7 : -p4)
aslur   	linseg  	0, irise, p7, ifall, 0	;make vibrato
;=====================
;VIBRATO DEPTH
;=====================
ioctvd	=	octpch(p5)
iminvd	=	octpch(8.02)
imaxvd	=	octpch(10.00)
irngvd	=	iminvd-imaxvd
indxvd	=	(ioctvd-iminvd)/irngvd
ivibd	tablei	indxvd, ivdfn, 1
avib	oscili	ivibd*2.5, p9, 25, 0.25	;vibrato depth, speed, table, phase
avib	=	avib+0.5
;=====================
;BREATH
;=====================
ioctbr	=	octpch(p5)
imnbr	=	octpch(8.02)
imxbr	=	octpch(10.00)
irngbr	=	imnbr-imxbr
indxbr	=	(ioctbr-imnbr)/irngbr
ibrth	tablei		indxbr, ibrthfn, 1
kenv1	linseg		0, .06, .5, .2, .9, p3-.16, .5, .02, 0
kenv2	linseg		0, .001, .1, p3-.002, .1, .001, 0
aflow	pinkish	kenv1*(p7*.02)
asum	=	(ibrth+aflow+kenv1)*avib
aslur   	=       	(aslur*avib)+asum
;=====================
;HIGH PARTIAL TRIM
;===================== 
ioct	=	octpch(p5)
imin	=	octpch(8.02)
imax	=	octpch(10.00)
irange	=	imin-imax
index	=	(ioct-imin)/irange
icut	tablei	index, icutfn, 1
;=====================
;PLAY 
;output to EQ instr 92
;=====================
play:					  ;waveform tables
aflu1	oscil3  		(amp+aslur), kpch, 5, i1		
aflu2	oscil3  		(amp+aslur), kpch, 6, i1
aflu3	oscil3  		(amp+aslur), kpch, 7, i1
aflu4	oscil3  		(amp+aslur), kpch, 8, i1
aflu5	oscil3  		(amp+aslur), kpch, 9, i1
aflu6	oscil3  		(amp+aslur), kpch, 10, i1
asig   	butterlp 	(aflu1+aflu2+aflu3+aflu4+aflu5+aflu6), icut, 1
	zawm		(asig*2.5)+asum*kenv2*kdclk, iout
	endin

	instr 6
;==========
;|||||||||| 	Bass			Total Range		G1-E4	(5.07-9.04)
;========== 		Range of Piece		G2-A3	(6.07-7.09)

idur    	=       	abs(p3)	;absolute duration value
ipch	=       	cpspch(p5) 
kpch    	=       	ipch
iamp    	=       	p4	;tied note starts at amp ramp
;=====================
;VARIABLE TABLES
;=====================
iatkfn	=	26	;attack	
idecfn	=	28	;decay
icutfn	=	32	;high partials trim
ivdfn	=	30	;vibrato depth
;=====================
;ATTACK
;=====================   
ioctat	=	octpch(p5)
iminat	=	octpch(7.09)
imaxat	=	octpch(6.07)
irngat	=	iminat-imaxat
indxat	=	(ioctat-iminat)/irngat
iatt	tablei	indxat, iatkfn, 1
;=====================
;DECAY
;=====================
ioctdc	=	octpch(p5)
imndc	=	octpch(6.07)
imxdc	=	octpch(7.09)
irngdc	=	imndc-imxdc
indxd	=	idur
indxdc	=	indxd+(ioctdc-imndc)/irngdc
idec	tablei	indxdc, idecfn, 1
;=====================
;PEAK/SWELL
;=====================
iout	=	6	;to EQ Instr 95
irise   	=       	idur*p7  ;0.1 = decresendo, 0.9 = crescendo, 0.5 = peak in middle
i1      	=       	-1         	;tied note phase
i2      	=       	-1          	;vibrato phase
kdclk	linseg  	0, .002, 1, p3-.004, 1, .002, 0	;declick envelope
;=====================
;AMPLITUDE RAMP 
;=====================
kamp   	linseg  	iamp, iatt, iamp, idur-0.6, iamp, idec, iamp
;=====================
;EXPRESSION 
;rise/fall, vibrato
;=====================
expr:
irise	=	(p7 >0 ? irise : iatt)
ifall	=	idur-irise
p6	=	((p6+p4) >0 ? p6 : -p4)
kslur   	linseg  	0, irise, p6, ifall, 0	;make vibrato
;=====================
;VIBRATO DEPTH
;=====================
ioctvd	=	octpch(p5)
iminvd	=	octpch(6.07)
imaxvd	=	octpch(7.09)
irngvd	=	iminvd-imaxvd
indxvd	=	(ioctvd-iminvd)/irngvd
ivibd	tablei	indxvd, ivdfn, 1
kvib	oscili	ivibd*3.5, p8, 25, 0.25	;vibrato depth, speed, table, phase
kvib	=	kvib+0.5
kslur	=	kslur*kvib
;=====================
;HIGH PARTIAL TRIM
;===================== 
ioct	=	octpch(p5)
imin	=	octpch(6.07)
imax	=	octpch(7.09)
irange	=	imin-imax
index	=	(ioct-imin)/irange
icut	tablei	index, icutfn, 1
;=====================
;PLAY 
;output to EQ instr
;=====================
play:			;waveform tables
abs1	pluck  		kamp+kslur, kpch, ipch, 19, 3, 0		
abs2	pluck  		kamp+kslur, kpch, ipch, 20, 3, 0
abs3	pluck  		kamp+kslur, kpch, ipch, 21, 3, 0
abs4	pluck  		kamp+kslur, kpch, ipch, 22, 3, 0
abs5	pluck  		kamp+kslur, kpch, ipch, 23, 3, 0
abs6	pluck  		kamp+kslur, kpch, ipch, 24, 3, 0
asig   	butterlp 	(abs1+abs2+abs3+abs4+abs5+abs6), icut, 1	;add tables, trim high partials
	zawm		(asig*1.5)*kdclk, iout
	endin
	
	instr   92
;==========
;|||||||||| 	EQ 1		Flute 1
;========== 
ilg1 		=	p4	;low gain1
ilg2 		=	p5	;low gain2
img 		=      	p6	;mid gain      
ihg1 		=       	p7	;high gain1
ihg2		=	p8	;high gain2
	
iin		=	p9
iout		=	p10	;to Reverb Instr 97 Left

ilc1		table	0, 33
ilc2		table	1, 33
ihc1		table	2, 33
ihc2		table	3, 33

asig		zar		iin
alsig1		butterlp	asig, ilc1		;low cut 1
alsig2		butterlp 	asig, ilc2	      	;low cut 2
atemp	 	butterhp 	asig, ilg2-ilc2/4		;mid range temp
amsig 		butterlp 	atemp, ihg1+ihc1/4	;mid cut
ahsig1 		butterhp 	asig, ihc1	       	;high cut 1
ahsig2 		butterhp 	asig, ihc2		;high cut 2

				;mac opcode multiplies & accumulates signals
aout		mac		ilg1, alsig1, ilg2, alsig2, img, amsig, ihg1, ahsig1, ihg2, ahsig2	
		zawm		aout, iout
		endin

	instr   93
;==========
;|||||||||| 	EQ 2		Flute 2
;========== 
ilg1 		=	p4	;low gain1
ilg2 		=	p5	;low gain2
img 		=      	p6	;mid gain      
ihg1 		=       	p7	;high gain1
ihg2		=	p8	;high gain2
	
iin		=	p9	
iout		=	p10	;to Reverb Instr 97 Right

ilc1		table	0, 34
ilc2		table	1, 34
ihc1		table	2, 34
ihc2		table	3, 34

asig		zar		iin
alsig1		butterlp	asig, ilc1		;low cut 1
alsig2		butterlp 	asig, ilc2	      	;low cut 2
atemp	 	butterhp 	asig, ilg2-ilc2/4		;mid range temp
amsig 		butterlp 	atemp, ihg1+ihc1/4	;mid cut
ahsig1 		butterhp 	asig, ihc1	       	;high cut 1
ahsig2 		butterhp 	asig, ihc2		;high cut 2

				;mac opcode multiplies & accumulates signals
aout		mac		ilg1, alsig1, ilg2, alsig2, img, amsig, ihg1, ahsig1, ihg2, ahsig2	
		zawm		aout, iout
		endin

	instr   94
;==========
;|||||||||| 	EQ 4		Bass
;========== 	

ilg1 		=	p4	;low gain1
ilg2 		=	p5	;low gain2
img 		=      	p6	;mid gain      
ihg1 		=       	p7	;high gain1
ihg2		=	p8	;high gain2
	
iin		=	p9
iout		=	p10	;to Reverb Instr 98 Right

ilc1		table	0, 35	
ilc2		table	1, 35	
ihc1		table	2, 35	
ihc2		table	3, 35

asig		zar		iin
alsig1		butterlp	asig, ilc1		;low cut 1
alsig2		butterlp 	asig, ilc2	      	;low cut 2
atemp	 	butterhp 	asig, ilg2-ilc2/4		;mid range temp
amsig 		butterlp 	atemp, ihg1+ihc1/4	;mid cut
ahsig1 		butterhp 	asig, ihc1	       	;high cut 1
ahsig2 		butterhp 	asig, ihc2		;high cut 2

				;mac opcode multiplies & accumulates signals
aout		mac		ilg1, alsig1, ilg2, alsig2, img, amsig, ihg1, ahsig1, ihg2, ahsig2	
		zawm		aout, iout
		endin

	instr 97	
;==========
;|||||||||| 	REVERB 1	Flute 1	Left
;==========	Flute 2	Right
iinL		=	p4
iinR		=	p5
ioutL		=	p6	;to Mixer Instr 100 Channel 3
ioutR		=	p7	;to Mixer Instr 100 Channel 4

irvbtimeL	=	p8
irvbtimeR	=	p9
iwetL		=	p10
idryL		=	1-p10
iwetR		=	p11
idryR		=	1-p11
ihfrollL	=	p12	;low pass filter: high frequency rolloff
ihfrollR	=	p13
ihfdiffL	=	p14	;high frequency diffusion
ihfdiffR	=	p15
kdclk		linseg  	0, .2, 1, p3-.4, 1, .2, 0	;declick envelope

asigL		zar	iinL	;Flute 1
asigR		zar	iinR	;Flute 2

aoutrevL	nreverb	asigL*iwetL, p8, p14
aoutrevR	nreverb	asigR*iwetR, p9, p15
aoutL		tone	aoutrevL, p12
aoutR		tone	aoutrevR, p13
		zawm	(aoutL+(asigL*idryL))*kdclk, ioutL
		zawm	(aoutR+(asigR*idryR))*kdclk, ioutR
		endin

	instr 98	
;==========
;|||||||||| 	REVERB 2	Bass
;==========	
iin		=	p4
iout		=	p5	;to Mixer Instr 100 Channel 6
irvbtime	=	p6
iwet		=	p7
idry		=	1-p7
ihfroll		=	p8	;low pass filter: high frequency rolloff
ihfdiff		=	p9	;high frequency diffusion
kdclk		linseg  	0, .2, 1, p3-.4, 1, .2, 0	;declick envelope
asig		zar	iin	;Bass
aoutrev	nreverb	asig*iwet, p6, p9
aout		tone	aoutrev, p8
		zawm	(aout+(asig*idry))*kdclk, iout
		endin

	instr 100
;==========
;|||||||||| MIXER		6 in, 2 out
;==========

;|||||||||| Gliss Instruments
;=====CHANNEL 1	=================================
a1		zar	p4		;IN	Gliss 1			 |
igLR1		init	p5		;GAIN				 |
;=====CHANNEL 2	=================================
a2		zar	p6		;IN	Gliss 2			 |
igLR2		init	p7		;GAIN				 |
;=====CHANNEL 3	=================================
a3		zar	p8		;IN	Gliss 3			 |
igLR3		init	p9		;GAIN				 |
;=================================================

;|||||||||| 2 Part Canon
;=====CHANNEL 4	=================================
a4		zar	p10		;IN	Flute 1			 |
igL4		init	p11*p12		;GAIN			 |
igR4		init	p11*(1-p12)	;PAN	1=left, 0=right		 |
;=====CHANNEL 5	=================================
a5		zar	p13		;IN	Flute 2			 |
igL5		init	p14*p15	;GAIN				 |
igR5		init	p14*(1-p15)	;PAN	1=left, 0=right		 |
;=================================================

;|||||||||| Bass
;=====CHANNEL 6	=================================
a6		zar	p16		;IN	Bass			 |
igL6		init	p17*p18	;GAIN				 |
igR6		init	p17*(1-p18)	;PAN	1=left, 0=right		 |
;=================================================

			;mac opcode multiplies & accumulates signals
aoutL		mac	igLR1, a1, igLR2, a2, igLR3, a3, igL4, a4, igL5, a5, igL6, a6 
aoutR		mac	igLR1, a1, igLR2, a2, igLR3, a3, igR4, a4, igR5, a5, igR6, a6 
		outs	aoutL, aoutR
		zacl	0, 20	;clear audio channels 0 to 20
		endin